#ifndef __AGAMEPT_H
#define __AGAMEPT_H

const int gpStick1 = 0x0001,
          gpStick2 = 0x0002,
          gpRCS    = 0x0004,
          gpWCS    = 0x0008;

const int STICK_1 = 0,
          STICK_2 = 1;

class AstGameport
{
  struct AstGamePortState
  {
    AstFixedpoint_t x, y;                // Current X and Y values
                                         // normalized from -1 to 1
    int cx, cy;                          // Center X and Y position
    int lx, ly;
    int rx, ry;
    int rawx, rawy;
    int dead_zone_lx, dead_zone_ly;      // Dead zone parameters
    int dead_zone_rx, dead_zone_ry;      //
    AstFixedpoint_t scalex_l, scalex_r;  // Adjustment values
    AstFixedpoint_t scaley_t, scaley_b;  //
  };

  protected:
    int              style, portmask;
    AstGamePortState stick[2];
    char             buttons;

    void AdjustForCalibration( void );
    void ReadPort( void );

    void CalibrateJoystick( void );
    void CalibrateRCS( void );
    void CalibrateFCS( void );
    void CalibrateWCS( void );

  public:
    AstGameport( int style );

    void            Calibrate( void );

    void            Update( void )
    {
       ReadPort();
       AdjustForCalibration();
    }


    char            GetButtons( void ) const { return buttons; }
    AstFixedpoint_t GetX( int stk ) const { return stick[stk].x; }
    AstFixedpoint_t GetY( int stk ) const { return stick[stk].y; }

    void            WaitForButtonPress( void );
    void            WaitForButtonRelease( void );
};

#endif
