#include "ast3d.h"

/*
** AstDrifter::AstDrifter( const char *n )
**
** This is an abstract class that supplies "drifting"
** capabilities to a derived class.  The constructor
** simply initializes its velocity vector by stopping.
*/
AstDrifter::AstDrifter( const char *n ) : AstObject( n )
{
   Stop();
}

/*
** void AstDrifter::CheckBoundaries( void )
**
** Drifting objects have the capability to drift outside of a
** playfield's bounds.  This function traps such out of bounds
** conditions and "bounces" the object by negating the
** offending element of the velocity vector.
*/
void AstDrifter::CheckBoundaries( void )
{
AstVector position = GetPositionWCS();

   if ( position.X < AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) )
   {
      position.X = AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) +
                   AstConvertToFix( 1 );
      velocity.X = -velocity.X;
   }
   else if ( position.X > AstConvertToFix( PLAY_FIELD_WIDTH/2 ) )
   {
      position.X = AstConvertToFix( PLAY_FIELD_WIDTH/2 ) -
                   AstConvertToFix( 1 );
      velocity.X = -velocity.X;
   }
   if ( position.Y < AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) )
   {
      position.Y = AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) +
                   AstConvertToFix( 1 );
      velocity.Y = -velocity.Y;
   }
   else if ( position.Y > AstConvertToFix( PLAY_FIELD_WIDTH/2 ) )
   {
      position.Y = AstConvertToFix( PLAY_FIELD_WIDTH/2 ) -
                   AstConvertToFix( 1 );
      velocity.Y = -velocity.Y;
   }
   if ( position.Z < AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) )
   {
      position.Z = AstConvertToFix( -PLAY_FIELD_WIDTH/2 ) +
                   AstConvertToFix( 1 );
      velocity.Z = -velocity.Z;
   }
   else if ( position.Z > AstConvertToFix( PLAY_FIELD_WIDTH/2 ) )
   {
      position.Z = AstConvertToFix( PLAY_FIELD_WIDTH/2 ) -
                   AstConvertToFix( 1 );
      velocity.Z = -velocity.Z;
   }
}

/*
** void AstDrifter::Drift( void )
**
** This is the core element of functionality that the
** AstDrifter class provides, simply consisting of adding the
** velocity vector's value to the object's location, then
** checking boundaries.
*/
void AstDrifter::Drift( void )
{
   MoveTo( GetPositionWCS() + velocity );
   CheckBoundaries();
}
