#include "ast3d.h"

static float starttime;

/*
** void AstLibInit( void )
**
** This initializes non-game specific code, such as the VGA
** card and trig tables.  This function should be called
** fairly early on (i.e. before most other initialization
** functions) because it may create or initialize variables
** that other routines are dependent on.
*/
void AstLibInit( void )
{
   /*
   ** Initialize the starttime variable so that we can
   ** calculate frames per second later.
   */
   starttime = clock();

   if ( ( trig_table = new AstTrigTable ) != 0 ) {
      if ( ( vga = new AstVGA ) == 0 ) {
         AstFatal( "Failed to allocate memory for VGA" );
      }
   }
   else {
      AstFatal( "Failed to allocate memory for trig table!" );
   }
}

/*
** void AstLibEnd( void )
**
** This is the complement to AstLibInit(), shutting down and
** unallocating memory that AstLibInit() claimed.  The VGA
** subsystem is also shutdown, reset, and freed.
*/
void AstLibEnd( void )
{
   if ( vga )
   {
      float fps, elapsed = clock() - starttime;

      vga->StopGraphics();
      fps = vga->GetFrames() / ( elapsed / ( float ) CLK_TCK );
      printf( "Frames per second: %2.2f\n", fps );
      delete vga;
      vga = 0;
   }
   if ( trig_table )
   {
      delete trig_table;
      trig_table = 0;
   }
}

/*
** void AstFatal( const char *s )
**
** This is AST3D's generic "uh-oh" function.  It shuts down
** the VGA system then dumps an error message out to the screen,
** then immediately aborts the program.  This function typically
** would only be called when some expected file could not be
** found or if there is a low memory situation.
*/
void AstFatal( const char *s )
{
   if ( vga != 0 )
      vga->StopGraphics();

  printf( "Fatal error: %s\n", s );
  exit( 1 );
}
