#include "ast3d.h"

/*
** void AstTriangle::ComputePlaneConstant( const AstVector *varray )
**
** The equation for a plane is:
**
** N dot P + D = 0
**
** Where N is the plane's surface normal and P is a point
** on the plane and D is the plane's plane constant.
**
** To compute the plane constant (used in our optimized backface
** removal algorithm) we solve for D:
**
** D = -( N dot P )
**
** N is computed as the cross product of two of triangle's sides, and
** one of the triangle's points is substituted for P.
*/
void AstTriangle::ComputePlaneConstant( const AstObjectDefinition &def )
{
AstVector side1, side2;

   side1 = def.GetVertex( vertices[1] ) - def.GetVertex( vertices[0] );
   side2 = def.GetVertex( vertices[2] ) - def.GetVertex( vertices[1] );

   surface_normal = side1.Cross( side2 );
   surface_normal.Normalize();
   plane_constant = -surface_normal.Dot( def.GetVertex( vertices[0] ) );
}
