#include "ast3d.h"

/*
** void AstMatrix4x3::Make?Rotation( unsigned tenths_degrees )
**
** Sets the rotation indices to the correct values to make
** this matrix a rotation matrix of about the given axis.
*/
void AstMatrix4x3::MakeXRotation( unsigned short tens_of_degrees )
{
unsigned short lookup = AstTrigTable::ConvertDegree( tens_of_degrees );

   data[1][1] =  trig_table->Cos( lookup );
   data[2][2] =  trig_table->Cos( lookup );
   data[2][1] = -trig_table->Sin( lookup );
   data[1][2] =  trig_table->Sin( lookup );
}
void AstMatrix4x3::MakeYRotation( unsigned short tens_of_degrees )
{
unsigned short lookup = AstTrigTable::ConvertDegree( tens_of_degrees );

   data[0][0] =  trig_table->Cos( lookup );
   data[0][2] = -trig_table->Sin( lookup );
   data[2][0] =  trig_table->Sin( lookup );
   data[2][2] =  trig_table->Cos( lookup );
}
void AstMatrix4x3::MakeZRotation( unsigned short tens_of_degrees )
{
unsigned short lookup = AstTrigTable::ConvertDegree( tens_of_degrees );

   data[0][0] =  trig_table->Cos( lookup );
   data[1][1] =  trig_table->Cos( lookup );
   data[0][1] =  trig_table->Sin( lookup );
   data[1][0] = -trig_table->Sin( lookup );
}

/*
** void AstMatrix4x3::Make?Translation( amount )
**
** Sets the correct matrix elements to make a translation matrix
** of "amount" units along the given axis.
*/
void AstMatrix4x3::MakeXTranslation( AstFixedpoint_t amount )
{
   data[3][0] = amount;
   data[3][1] = 0;
   data[3][2] = 0;
}
void AstMatrix4x3::MakeYTranslation( AstFixedpoint_t amount )
{
   data[3][0] = 0;
   data[3][1] = amount;
   data[3][2] = 0;
}
void AstMatrix4x3::MakeZTranslation( AstFixedpoint_t amount )
{
   data[3][0] = 0;
   data[3][1] = 0;
   data[3][2] = amount;
}

/*
** Private identity matrix data set
*/
const static AstFixedpoint_t _identity_matrix_4x3[4][3] = {
   { AstConvertToFix( 1 ), 0L, 0L },
   { 0L, AstConvertToFix( 1 ), 0L },
   { 0L, 0L, AstConvertToFix( 1 ) },
   { 0L, 0L, 0L}
};

/*
** void AstMatrix4x3::MakeIdentity( void )
**
** Makes the matrix into an identity matrix by copying the identity
** matrix source data values into its own data set.
*/
void AstMatrix4x3::MakeIdentity( void )
{
   memcpy( data, _identity_matrix_4x3, sizeof( _identity_matrix_4x3 ) );
}

/*
** AstMatrix4x3 operator * ( const AstMatrix4x3 &a, const AstMatrix4x3 &b )
**
** Overloaded * operator that multiplies two matrices together.
*/
AstMatrix4x3 operator * ( const AstMatrix4x3 &a, const AstMatrix4x3 &b )
{
static AstMatrix4x3 m;

   /*
   **   m[0][0] = a[0][0] * b[0][0] +
   **             a[0][1] * b[1][0] +
   **             a[0][2] * b[2][0] +
   **           ( a[0][3] * b[3][0] ) = 0;
   */
   m[0][0] = AstFixMulFix( a[0][0], b[0][0] ) +
             AstFixMulFix( a[0][1], b[1][0] ) +
             AstFixMulFix( a[0][2], b[2][0] );

   /*
   **   m[0][1] = a[0][0] * b[0][1] +
   **             a[0][1] * b[1][1] +
   **             a[0][2] * b[2][1] +
   **           ( a[0][3] * b[3][1] ) = 0;
   */
   m[0][1] = AstFixMulFix( a[0][0], b[0][1] ) +
             AstFixMulFix( a[0][1], b[1][1] ) +
             AstFixMulFix( a[0][2], b[2][1] );

   /*
   **   m[0][2] = a[0][0] * b[0][2] +
   **             a[0][1] * b[1][2] +
   **             a[0][2] * b[2][2] +
   **           ( a[0][3] * b[3][2] ) = 0;
   */
   m[0][2] = AstFixMulFix( a[0][0], b[0][2] ) +
             AstFixMulFix( a[0][1], b[1][2] ) +
             AstFixMulFix( a[0][2], b[2][2] );

   /*
   **   m[1][0] = a[1][0] * b[0][0] +
   **             a[1][1] * b[1][0] +
   **             a[1][2] * b[2][0] +
   **           ( a[1][3] * b[3][0] ) = 0;
   */
   m[1][0] = AstFixMulFix( a[1][0], b[0][0] ) +
             AstFixMulFix( a[1][1], b[1][0] ) +
             AstFixMulFix( a[1][2], b[2][0] );

   /*
   **   m[1][1] = a[1][0] * b[0][1] +
   **             a[1][1] * b[1][1] +
   **             a[1][2] * b[2][1] +
   **           ( a[1][3] * b[3][1] ) = 0;
   */
   m[1][1] = AstFixMulFix( a[1][0], b[0][1] ) +
             AstFixMulFix( a[1][1], b[1][1] ) +
             AstFixMulFix( a[1][2], b[2][1] );

   /*
   **   m[1][2] = a[1][0] * b[0][2] +
   **             a[1][1] * b[1][2] +
   **             a[1][2] * b[2][2] +
   **           ( a[1][3] * b[3][2] ) = 0;
   */
   m[1][2] = AstFixMulFix( a[1][0], b[0][2] ) +
             AstFixMulFix( a[1][1], b[1][2] ) +
             AstFixMulFix( a[1][2], b[2][2] );

   /*
   **   m[2][0] = a[2][0] * b[0][0] +
   **             a[2][1] * b[1][0] +
   **             a[2][2] * b[2][0] +
   **           ( a[2][3] * b[3][0] ) = 0;
   */
   m[2][0] = AstFixMulFix( a[2][0], b[0][0] ) +
             AstFixMulFix( a[2][1], b[1][0] ) +
             AstFixMulFix( a[2][2], b[2][0] );

   /*
   **   m[2][1] = a[2][0] * b[0][1] +
   **             a[2][1] * b[1][1] +
   **             a[2][2] * b[2][1] +
   **           ( a[2][3] * b[3][1] ) = 0;
   */
   m[2][1] = AstFixMulFix( a[2][0], b[0][1] ) +
             AstFixMulFix( a[2][1], b[1][1] ) +
             AstFixMulFix( a[2][2], b[2][1] );

   /*
   **   m[2][2] = a[2][0] * b[0][2] +
   **             a[2][1] * b[1][2] +
   **             a[2][2] * b[2][2] +
   **           ( a[2][3] * b[3][2] ) = 0;
   */
   m[2][2] = AstFixMulFix( a[2][0], b[0][2] ) +
             AstFixMulFix( a[2][1], b[1][2] ) +
             AstFixMulFix( a[2][2], b[2][2] );

   /*
   **   m[3][0] = a[3][0] * b[0][0] +
   **             a[3][1] * b[1][0] +
   **             a[3][2] * b[2][0] +
   **           ( a[3][3] * b[3][0] ) = b[3][0];
   */
   m[3][0] = AstFixMulFix( a[3][0], b[0][0] ) +
             AstFixMulFix( a[3][1], b[1][0] ) +
             AstFixMulFix( a[3][2], b[2][0] ) +
             b[3][0];

   /*
   **   m[3][1] = a[3][0] * b[0][1] +
   **             a[3][1] * b[1][1] +
   **             a[3][2] * b[2][1] +
   **           ( a[3][3] * b[3][1] ) = b[3][1];
   */
   m[3][1] = AstFixMulFix( a[3][0], b[0][1] ) +
             AstFixMulFix( a[3][1], b[1][1] ) +
             AstFixMulFix( a[3][2], b[2][1] ) +
             b[3][1];

   /*
   **   m[3][2] = a[3][0] * b[0][2] +
   **             a[3][1] * b[1][2] +
   **             a[3][2] * b[2][2] +
   **           ( a[3][3] * b[3][2] ) = b[3][2];
   */
   m[3][2] = AstFixMulFix( a[3][0], b[0][2] ) +
             AstFixMulFix( a[3][1], b[1][2] ) +
             AstFixMulFix( a[3][2], b[2][2] ) +
             b[3][2];

   return m;
}

/*
** void AstMatrix4x3::GetRotationMatrix( AstMatrix3x3 &m ) const
**
** This routine copies the rotation submatrix of the matrix into
** the 3x3 rotation matrix passed to the function.
*/
void AstMatrix4x3::GetRotationMatrix( AstMatrix3x3 &m ) const
{
   for ( int i = 0; i < 3; i++ )
      memcpy( m[i], data[i], sizeof( AstFixedpoint_t ) * 3 );
}

/*
** void AstMatrix4x3::SetRotationMatrix( const AstMatrix3x3 &m )
**
** Sets the rotation submatrix of the given matrix to the 3x3
** rotation matrix passed to this member function.
*/
void AstMatrix4x3::SetRotationMatrix( const AstMatrix3x3 &m )
{
   for ( int i = 0; i < 3; i++ )
      memcpy( data[i], m[i], sizeof( AstFixedpoint_t ) * 3 );
}

/*
** void AstMatrix4x3::NormalizeDirectionVectors( void )
**
** This function normalizes the direction vectors of the matrix,
** preventing unwanted spurious scaling from creeping in as a
** a result of accumulated error.
*/
void AstMatrix4x3::NormalizeDirectionVectors( void )
{
AstVector v;

   GetVU( v );
   v.Normalize();
   SetVU( v );

   GetVR( v );
   v.Normalize();
   SetVR( v );

   GetVF( v );
   v.Normalize();
   SetVF( v );
}

